from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.Geom2dAdaptor import *
from OCC.Core.Geom2d import *
from OCC.Core.gp import *
from OCC.Core.GccAna import *
from OCC.Core.GccEnt import *
from OCC.Core.math import *


class Geom2dGcc_Type2(IntEnum):
	Geom2dGcc_CuCuOnCu: int = ...
	Geom2dGcc_CiCuOnCu: int = ...
	Geom2dGcc_LiCuOnCu: int = ...
	Geom2dGcc_CuPtOnCu: int = ...
	Geom2dGcc_CuCuOnLi: int = ...
	Geom2dGcc_CiCuOnLi: int = ...
	Geom2dGcc_LiCuOnLi: int = ...
	Geom2dGcc_CuPtOnLi: int = ...
	Geom2dGcc_CuCuOnCi: int = ...
	Geom2dGcc_CiCuOnCi: int = ...
	Geom2dGcc_LiCuOnCi: int = ...
	Geom2dGcc_CuPtOnCi: int = ...
Geom2dGcc_CuCuOnCu = Geom2dGcc_Type2.Geom2dGcc_CuCuOnCu
Geom2dGcc_CiCuOnCu = Geom2dGcc_Type2.Geom2dGcc_CiCuOnCu
Geom2dGcc_LiCuOnCu = Geom2dGcc_Type2.Geom2dGcc_LiCuOnCu
Geom2dGcc_CuPtOnCu = Geom2dGcc_Type2.Geom2dGcc_CuPtOnCu
Geom2dGcc_CuCuOnLi = Geom2dGcc_Type2.Geom2dGcc_CuCuOnLi
Geom2dGcc_CiCuOnLi = Geom2dGcc_Type2.Geom2dGcc_CiCuOnLi
Geom2dGcc_LiCuOnLi = Geom2dGcc_Type2.Geom2dGcc_LiCuOnLi
Geom2dGcc_CuPtOnLi = Geom2dGcc_Type2.Geom2dGcc_CuPtOnLi
Geom2dGcc_CuCuOnCi = Geom2dGcc_Type2.Geom2dGcc_CuCuOnCi
Geom2dGcc_CiCuOnCi = Geom2dGcc_Type2.Geom2dGcc_CiCuOnCi
Geom2dGcc_LiCuOnCi = Geom2dGcc_Type2.Geom2dGcc_LiCuOnCi
Geom2dGcc_CuPtOnCi = Geom2dGcc_Type2.Geom2dGcc_CuPtOnCi

class Geom2dGcc_Type3(IntEnum):
	Geom2dGcc_CuCu: int = ...
	Geom2dGcc_CiCu: int = ...
Geom2dGcc_CuCu = Geom2dGcc_Type3.Geom2dGcc_CuCu
Geom2dGcc_CiCu = Geom2dGcc_Type3.Geom2dGcc_CiCu

class Geom2dGcc_Type1(IntEnum):
	Geom2dGcc_CuCuCu: int = ...
	Geom2dGcc_CiCuCu: int = ...
	Geom2dGcc_CiCiCu: int = ...
	Geom2dGcc_CiLiCu: int = ...
	Geom2dGcc_LiLiCu: int = ...
	Geom2dGcc_LiCuCu: int = ...
Geom2dGcc_CuCuCu = Geom2dGcc_Type1.Geom2dGcc_CuCuCu
Geom2dGcc_CiCuCu = Geom2dGcc_Type1.Geom2dGcc_CiCuCu
Geom2dGcc_CiCiCu = Geom2dGcc_Type1.Geom2dGcc_CiCiCu
Geom2dGcc_CiLiCu = Geom2dGcc_Type1.Geom2dGcc_CiLiCu
Geom2dGcc_LiLiCu = Geom2dGcc_Type1.Geom2dGcc_LiLiCu
Geom2dGcc_LiCuCu = Geom2dGcc_Type1.Geom2dGcc_LiCuCu

class geom2dgcc:
	@staticmethod
	def Enclosed(Obj: Geom2dAdaptor_Curve) -> Geom2dGcc_QualifiedCurve: ...
	@staticmethod
	def Enclosing(Obj: Geom2dAdaptor_Curve) -> Geom2dGcc_QualifiedCurve: ...
	@staticmethod
	def Outside(Obj: Geom2dAdaptor_Curve) -> Geom2dGcc_QualifiedCurve: ...
	@staticmethod
	def Unqualified(Obj: Geom2dAdaptor_Curve) -> Geom2dGcc_QualifiedCurve: ...

class Geom2dGcc_Circ2d2TanOn:
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Qualified2: Geom2dGcc_QualifiedCurve, OnCurve: Geom2dAdaptor_Curve, Tolerance: float, Param1: float, Param2: float, ParamOn: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Point: Geom2d_Point, OnCurve: Geom2dAdaptor_Curve, Tolerance: float, Param1: float, ParamOn: float) -> None: ...
	@overload
	def __init__(self, Point1: Geom2d_Point, Point2: Geom2d_Point, OnCurve: Geom2dAdaptor_Curve, Tolerance: float) -> None: ...
	def CenterOn3(self, Index: int, PntSol: gp_Pnt2d) -> float: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self, Index: int) -> bool: ...
	def IsTheSame2(self, Index: int) -> bool: ...
	def NbSolutions(self) -> int: ...
	@overload
	def Results(self, Circ: GccAna_Circ2d2TanOn) -> None: ...
	@overload
	def Results(self, Circ: Geom2dGcc_Circ2d2TanOnGeo) -> None: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2d2TanOnGeo:
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: GccEnt_QualifiedCirc, OnCurv: Geom2dAdaptor_Curve, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: GccEnt_QualifiedLin, OnCurv: Geom2dAdaptor_Curve, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Point2: gp_Pnt2d, OnCurv: Geom2dAdaptor_Curve, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: GccEnt_QualifiedLin, OnCurv: Geom2dAdaptor_Curve, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: gp_Pnt2d, OnCurv: Geom2dAdaptor_Curve, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Point1: gp_Pnt2d, Point2: gp_Pnt2d, OnCurv: Geom2dAdaptor_Curve, Tolerance: float) -> None: ...
	def CenterOn3(self, Index: int, PntSol: gp_Pnt2d) -> float: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self, Index: int) -> bool: ...
	def IsTheSame2(self, Index: int) -> bool: ...
	def NbSolutions(self) -> int: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2d2TanOnIter:
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: Geom2dGcc_QCurve, OnLine: gp_Lin2d, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: Geom2dGcc_QCurve, OnLine: gp_Lin2d, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Qualified2: Geom2dGcc_QCurve, OnLine: gp_Lin2d, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Point2: gp_Pnt2d, OnLine: gp_Lin2d, Param1: float, Param2: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: Geom2dGcc_QCurve, OnCirc: gp_Circ2d, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: Geom2dGcc_QCurve, OnCirc: gp_Circ2d, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Qualified2: Geom2dGcc_QCurve, OnCirc: gp_Circ2d, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Point2: gp_Pnt2d, OnCirc: gp_Circ2d, Param1: float, Param2: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: Geom2dGcc_QCurve, OnCurv: Geom2dAdaptor_Curve, Param1: float, Param2: float, ParamOn: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: Geom2dGcc_QCurve, OnCurve: Geom2dAdaptor_Curve, Param1: float, Param2: float, ParamOn: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Point2: gp_Pnt2d, OnCurve: Geom2dAdaptor_Curve, Param1: float, ParamOn: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Qualified2: Geom2dGcc_QCurve, OnCurve: Geom2dAdaptor_Curve, Param1: float, Param2: float, ParamOn: float, Tolerance: float) -> None: ...
	def CenterOn3(self, PntSol: gp_Pnt2d) -> float: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self) -> bool: ...
	def IsTheSame2(self) -> bool: ...
	def Tangency1(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self) -> gp_Circ2d: ...
	def WhichQualifier(self, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2d2TanRad:
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Qualified2: Geom2dGcc_QualifiedCurve, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Point: Geom2d_Point, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Point1: Geom2d_Point, Point2: Geom2d_Point, Radius: float, Tolerance: float) -> None: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self, Index: int) -> bool: ...
	def IsTheSame2(self, Index: int) -> bool: ...
	def NbSolutions(self) -> int: ...
	@overload
	def Results(self, Circ: GccAna_Circ2d2TanRad) -> None: ...
	@overload
	def Results(self, Circ: Geom2dGcc_Circ2d2TanRadGeo) -> None: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2d2TanRadGeo:
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: Geom2dGcc_QCurve, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: Geom2dGcc_QCurve, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Qualified2: Geom2dGcc_QCurve, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Point2: gp_Pnt2d, Radius: float, Tolerance: float) -> None: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self, Index: int) -> bool: ...
	def IsTheSame2(self, Index: int) -> bool: ...
	def NbSolutions(self) -> int: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2d3Tan:
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Qualified2: Geom2dGcc_QualifiedCurve, Qualified3: Geom2dGcc_QualifiedCurve, Tolerance: float, Param1: float, Param2: float, Param3: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Qualified2: Geom2dGcc_QualifiedCurve, Point: Geom2d_Point, Tolerance: float, Param1: float, Param2: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Point1: Geom2d_Point, Point2: Geom2d_Point, Tolerance: float, Param1: float) -> None: ...
	@overload
	def __init__(self, Point1: Geom2d_Point, Point2: Geom2d_Point, Point3: Geom2d_Point, Tolerance: float) -> None: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self, Index: int) -> bool: ...
	def IsTheSame2(self, Index: int) -> bool: ...
	def IsTheSame3(self, Index: int) -> bool: ...
	def NbSolutions(self) -> int: ...
	def Results(self, Circ: GccAna_Circ2d3Tan, Rank1: int, Rank2: int, Rank3: int) -> None: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency3(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position, Qualif3: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2d3TanIter:
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: GccEnt_QualifiedCirc, Qualified3: Geom2dGcc_QCurve, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: Geom2dGcc_QCurve, Qualified3: Geom2dGcc_QCurve, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: GccEnt_QualifiedLin, Qualified3: Geom2dGcc_QCurve, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: Geom2dGcc_QCurve, Point3: gp_Pnt2d, Param1: float, Param2: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: GccEnt_QualifiedLin, Qualified3: Geom2dGcc_QCurve, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: Geom2dGcc_QCurve, Qualified3: Geom2dGcc_QCurve, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, Qualified2: Geom2dGcc_QCurve, Point3: gp_Pnt2d, Param1: float, Param2: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Point1: gp_Pnt2d, Point2: gp_Pnt2d, Param1: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Qualified2: Geom2dGcc_QCurve, Point2: gp_Pnt2d, Param1: float, Param2: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Qualified2: Geom2dGcc_QCurve, Qualified3: Geom2dGcc_QCurve, Param1: float, Param2: float, Param3: float, Tolerance: float) -> None: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self) -> bool: ...
	def IsTheSame2(self) -> bool: ...
	def IsTheSame3(self) -> bool: ...
	def Tangency1(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency3(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self) -> gp_Circ2d: ...
	def WhichQualifier(self, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position, Qualif3: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2dTanCen:
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Pcenter: Geom2d_Point, Tolerance: float) -> None: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self, Index: int) -> bool: ...
	def NbSolutions(self) -> int: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2dTanCenGeo:
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Pcenter: gp_Pnt2d, Tolerance: float) -> None: ...
	def IsDone(self) -> bool: ...
	def NbSolutions(self) -> int: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2dTanOnRad:
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, OnCurv: Geom2dAdaptor_Curve, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Point1: Geom2d_Point, OnCurv: Geom2dAdaptor_Curve, Radius: float, Tolerance: float) -> None: ...
	def CenterOn3(self, Index: int, PntSol: gp_Pnt2d) -> float: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self, Index: int) -> bool: ...
	def NbSolutions(self) -> int: ...
	@overload
	def Results(self, Circ: GccAna_Circ2dTanOnRad) -> None: ...
	@overload
	def Results(self, Circ: Geom2dGcc_Circ2dTanOnRadGeo) -> None: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position) -> None: ...

class Geom2dGcc_Circ2dTanOnRadGeo:
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, OnLine: gp_Lin2d, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, OnCirc: gp_Circ2d, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, OnCurv: Geom2dAdaptor_Curve, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedLin, OnCurv: Geom2dAdaptor_Curve, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, OnCurv: Geom2dAdaptor_Curve, Radius: float, Tolerance: float) -> None: ...
	@overload
	def __init__(self, Point1: gp_Pnt2d, OnCurv: Geom2dAdaptor_Curve, Radius: float, Tolerance: float) -> None: ...
	def CenterOn3(self, Index: int, PntSol: gp_Pnt2d) -> float: ...
	def IsDone(self) -> bool: ...
	def IsTheSame1(self, Index: int) -> bool: ...
	def NbSolutions(self) -> int: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Circ2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position) -> None: ...

class Geom2dGcc_CurveTool:
	@staticmethod
	def D1(C: Geom2dAdaptor_Curve, U: float, P: gp_Pnt2d, T: gp_Vec2d) -> None: ...
	@staticmethod
	def D2(C: Geom2dAdaptor_Curve, U: float, P: gp_Pnt2d, T: gp_Vec2d, N: gp_Vec2d) -> None: ...
	@staticmethod
	def D3(C: Geom2dAdaptor_Curve, U: float, P: gp_Pnt2d, T: gp_Vec2d, N: gp_Vec2d, dN: gp_Vec2d) -> None: ...
	@staticmethod
	def EpsX(C: Geom2dAdaptor_Curve, Tol: float) -> float: ...
	@staticmethod
	def FirstParameter(C: Geom2dAdaptor_Curve) -> float: ...
	@staticmethod
	def LastParameter(C: Geom2dAdaptor_Curve) -> float: ...
	@staticmethod
	def NbSamples(C: Geom2dAdaptor_Curve) -> int: ...
	@staticmethod
	def Value(C: Geom2dAdaptor_Curve, X: float) -> gp_Pnt2d: ...

class Geom2dGcc_FunctionTanCirCu(math_FunctionWithDerivative):
	def __init__(self, Circ: gp_Circ2d, Curv: Geom2dAdaptor_Curve) -> None: ...
	def Derivative(self, X: float) -> Tuple[bool, float]: ...
	def Value(self, X: float) -> Tuple[bool, float]: ...
	def Values(self, X: float) -> Tuple[bool, float, float]: ...

class Geom2dGcc_FunctionTanCuCu(math_FunctionSetWithDerivatives):
	@overload
	def __init__(self, Curv1: Geom2dAdaptor_Curve, Curv2: Geom2dAdaptor_Curve) -> None: ...
	@overload
	def __init__(self, Circ1: gp_Circ2d, Curv2: Geom2dAdaptor_Curve) -> None: ...
	def Derivatives(self, X: math_Vector, Deriv: math_Matrix) -> bool: ...
	def InitDerivative(self, X: math_Vector, Point1: gp_Pnt2d, Point2: gp_Pnt2d, Tan1: gp_Vec2d, Tan2: gp_Vec2d, D21: gp_Vec2d, D22: gp_Vec2d) -> None: ...
	def NbEquations(self) -> int: ...
	def NbVariables(self) -> int: ...
	def Value(self, X: math_Vector, F: math_Vector) -> bool: ...
	def Values(self, X: math_Vector, F: math_Vector, Deriv: math_Matrix) -> bool: ...

class Geom2dGcc_FunctionTanCuCuOnCu(math_FunctionSetWithDerivatives):
	@overload
	def __init__(self, C1: Geom2dAdaptor_Curve, C2: Geom2dAdaptor_Curve, OnCi: gp_Circ2d, Rad: float) -> None: ...
	@overload
	def __init__(self, C1: gp_Circ2d, C2: Geom2dAdaptor_Curve, OnCi: gp_Circ2d, Rad: float) -> None: ...
	@overload
	def __init__(self, L1: gp_Lin2d, C2: Geom2dAdaptor_Curve, OnCi: gp_Circ2d, Rad: float) -> None: ...
	@overload
	def __init__(self, C1: Geom2dAdaptor_Curve, P2: gp_Pnt2d, OnCi: gp_Circ2d, Rad: float) -> None: ...
	@overload
	def __init__(self, C1: Geom2dAdaptor_Curve, C2: Geom2dAdaptor_Curve, OnLi: gp_Lin2d, Rad: float) -> None: ...
	@overload
	def __init__(self, C1: gp_Circ2d, C2: Geom2dAdaptor_Curve, OnLi: gp_Lin2d, Rad: float) -> None: ...
	@overload
	def __init__(self, L1: gp_Lin2d, C2: Geom2dAdaptor_Curve, OnLi: gp_Lin2d, Rad: float) -> None: ...
	@overload
	def __init__(self, C1: Geom2dAdaptor_Curve, P2: gp_Pnt2d, OnLi: gp_Lin2d, Rad: float) -> None: ...
	@overload
	def __init__(self, C1: Geom2dAdaptor_Curve, C2: Geom2dAdaptor_Curve, OnCu: Geom2dAdaptor_Curve, Rad: float) -> None: ...
	@overload
	def __init__(self, C1: gp_Circ2d, C2: Geom2dAdaptor_Curve, OnCu: Geom2dAdaptor_Curve, Rad: float) -> None: ...
	@overload
	def __init__(self, L1: gp_Lin2d, C2: Geom2dAdaptor_Curve, OnCu: Geom2dAdaptor_Curve, Rad: float) -> None: ...
	@overload
	def __init__(self, C1: Geom2dAdaptor_Curve, P1: gp_Pnt2d, OnCu: Geom2dAdaptor_Curve, Rad: float) -> None: ...
	def Derivatives(self, X: math_Vector, D: math_Matrix) -> bool: ...
	def InitDerivative(self, X: math_Vector, Point1: gp_Pnt2d, Point2: gp_Pnt2d, Point3: gp_Pnt2d, Tan1: gp_Vec2d, Tan2: gp_Vec2d, Tan3: gp_Vec2d, D21: gp_Vec2d, D22: gp_Vec2d, D23: gp_Vec2d) -> None: ...
	def NbEquations(self) -> int: ...
	def NbVariables(self) -> int: ...
	def Value(self, X: math_Vector, F: math_Vector) -> bool: ...
	def Values(self, X: math_Vector, F: math_Vector, D: math_Matrix) -> bool: ...

class Geom2dGcc_FunctionTanCuPnt(math_FunctionWithDerivative):
	def __init__(self, C: Geom2dAdaptor_Curve, Point: gp_Pnt2d) -> None: ...
	def Derivative(self, X: float) -> Tuple[bool, float]: ...
	def Value(self, X: float) -> Tuple[bool, float]: ...
	def Values(self, X: float) -> Tuple[bool, float, float]: ...

class Geom2dGcc_FunctionTanObl(math_FunctionWithDerivative):
	def __init__(self, Curve: Geom2dAdaptor_Curve, Dir: gp_Dir2d) -> None: ...
	def Derivative(self, X: float) -> Tuple[bool, float]: ...
	def Value(self, X: float) -> Tuple[bool, float]: ...
	def Values(self, X: float) -> Tuple[bool, float, float]: ...

class Geom2dGcc_Lin2d2Tan:
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Qualified2: Geom2dGcc_QualifiedCurve, Tolang: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, ThePoint: gp_Pnt2d, Tolang: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, Qualified2: Geom2dGcc_QualifiedCurve, Tolang: float, Param1: float, Param2: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, ThePoint: gp_Pnt2d, Tolang: float, Param1: float) -> None: ...
	def IsDone(self) -> bool: ...
	def NbSolutions(self) -> int: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Lin2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position) -> None: ...

class Geom2dGcc_Lin2d2TanIter:
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, ThePoint: gp_Pnt2d, Param1: float, Tolang: float) -> None: ...
	@overload
	def __init__(self, Qualified1: GccEnt_QualifiedCirc, Qualified2: Geom2dGcc_QCurve, Param2: float, Tolang: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QCurve, Qualified2: Geom2dGcc_QCurve, Param1: float, Param2: float, Tolang: float) -> None: ...
	def IsDone(self) -> bool: ...
	def Tangency1(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def Tangency2(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self) -> gp_Lin2d: ...
	def WhichQualifier(self, Qualif1: GccEnt_Position, Qualif2: GccEnt_Position) -> None: ...

class Geom2dGcc_Lin2dTanObl:
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, TheLin: gp_Lin2d, TolAng: float, Angle: float) -> None: ...
	@overload
	def __init__(self, Qualified1: Geom2dGcc_QualifiedCurve, TheLin: gp_Lin2d, TolAng: float, Param1: float, Angle: float) -> None: ...
	def Intersection2(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def IsDone(self) -> bool: ...
	def NbSolutions(self) -> int: ...
	def Tangency1(self, Index: int, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self, Index: int) -> gp_Lin2d: ...
	def WhichQualifier(self, Index: int, Qualif1: GccEnt_Position) -> None: ...

class Geom2dGcc_Lin2dTanOblIter:
	def __init__(self, Qualified1: Geom2dGcc_QCurve, TheLin: gp_Lin2d, Param1: float, TolAng: float, Angle: Optional[float] = 0) -> None: ...
	def Intersection2(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def IsDone(self) -> bool: ...
	def IsParallel2(self) -> bool: ...
	def Tangency1(self, PntSol: gp_Pnt2d) -> Tuple[float, float]: ...
	def ThisSolution(self) -> gp_Lin2d: ...
	def WhichQualifier(self, Qualif1: GccEnt_Position) -> None: ...

class Geom2dGcc_QCurve:
	def __init__(self, Curve: Geom2dAdaptor_Curve, Qualifier: GccEnt_Position) -> None: ...
	def IsEnclosed(self) -> bool: ...
	def IsEnclosing(self) -> bool: ...
	def IsOutside(self) -> bool: ...
	def IsUnqualified(self) -> bool: ...
	def Qualified(self) -> Geom2dAdaptor_Curve: ...
	def Qualifier(self) -> GccEnt_Position: ...

class Geom2dGcc_QualifiedCurve:
	def __init__(self, Curve: Geom2dAdaptor_Curve, Qualifier: GccEnt_Position) -> None: ...
	def IsEnclosed(self) -> bool: ...
	def IsEnclosing(self) -> bool: ...
	def IsOutside(self) -> bool: ...
	def IsUnqualified(self) -> bool: ...
	def Qualified(self) -> Geom2dAdaptor_Curve: ...
	def Qualifier(self) -> GccEnt_Position: ...

#classnotwrapped
class Geom2dGcc_FunctionTanCuCuCu: ...

# harray1 classes
# harray2 classes
# hsequence classes

geom2dgcc_Enclosed = geom2dgcc.Enclosed
geom2dgcc_Enclosing = geom2dgcc.Enclosing
geom2dgcc_Outside = geom2dgcc.Outside
geom2dgcc_Unqualified = geom2dgcc.Unqualified
Geom2dGcc_CurveTool_D1 = Geom2dGcc_CurveTool.D1
Geom2dGcc_CurveTool_D2 = Geom2dGcc_CurveTool.D2
Geom2dGcc_CurveTool_D3 = Geom2dGcc_CurveTool.D3
Geom2dGcc_CurveTool_EpsX = Geom2dGcc_CurveTool.EpsX
Geom2dGcc_CurveTool_FirstParameter = Geom2dGcc_CurveTool.FirstParameter
Geom2dGcc_CurveTool_LastParameter = Geom2dGcc_CurveTool.LastParameter
Geom2dGcc_CurveTool_NbSamples = Geom2dGcc_CurveTool.NbSamples
Geom2dGcc_CurveTool_Value = Geom2dGcc_CurveTool.Value



from enum import IntEnum
from typing import overload, NewType, Optional, Tuple

from OCC.Core.Standard import *
from OCC.Core.NCollection import *
from OCC.Core.StepData import *
from OCC.Core.Interface import *
from OCC.Core.StepShape import *


class RWStepShape_RWAdvancedBrepShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_AdvancedBrepShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_AdvancedBrepShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_AdvancedBrepShapeRepresentation) -> None: ...

class RWStepShape_RWAdvancedFace:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_AdvancedFace) -> None: ...
	def Share(self, ent: StepShape_AdvancedFace, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_AdvancedFace) -> None: ...

class RWStepShape_RWAngularLocation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_AngularLocation) -> None: ...
	def Share(self, ent: StepShape_AngularLocation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_AngularLocation) -> None: ...

class RWStepShape_RWAngularSize:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_AngularSize) -> None: ...
	def Share(self, ent: StepShape_AngularSize, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_AngularSize) -> None: ...

class RWStepShape_RWBlock:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Block) -> None: ...
	def Share(self, ent: StepShape_Block, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Block) -> None: ...

class RWStepShape_RWBooleanResult:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_BooleanResult) -> None: ...
	def Share(self, ent: StepShape_BooleanResult, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_BooleanResult) -> None: ...

class RWStepShape_RWBoxDomain:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_BoxDomain) -> None: ...
	def Share(self, ent: StepShape_BoxDomain, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_BoxDomain) -> None: ...

class RWStepShape_RWBoxedHalfSpace:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_BoxedHalfSpace) -> None: ...
	def Share(self, ent: StepShape_BoxedHalfSpace, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_BoxedHalfSpace) -> None: ...

class RWStepShape_RWBrepWithVoids:
	def __init__(self) -> None: ...
	def Check(self, ent: StepShape_BrepWithVoids, shares: Interface_ShareTool, ach: Interface_Check) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_BrepWithVoids) -> None: ...
	def Share(self, ent: StepShape_BrepWithVoids, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_BrepWithVoids) -> None: ...

class RWStepShape_RWClosedShell:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ClosedShell) -> None: ...
	def Share(self, ent: StepShape_ClosedShell, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ClosedShell) -> None: ...

class RWStepShape_RWCompoundShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_CompoundShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_CompoundShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_CompoundShapeRepresentation) -> None: ...

class RWStepShape_RWConnectedEdgeSet:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ConnectedEdgeSet) -> None: ...
	def Share(self, ent: StepShape_ConnectedEdgeSet, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ConnectedEdgeSet) -> None: ...

class RWStepShape_RWConnectedFaceSet:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ConnectedFaceSet) -> None: ...
	def Share(self, ent: StepShape_ConnectedFaceSet, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ConnectedFaceSet) -> None: ...

class RWStepShape_RWConnectedFaceShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ConnectedFaceShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_ConnectedFaceShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ConnectedFaceShapeRepresentation) -> None: ...

class RWStepShape_RWConnectedFaceSubSet:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ConnectedFaceSubSet) -> None: ...
	def Share(self, ent: StepShape_ConnectedFaceSubSet, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ConnectedFaceSubSet) -> None: ...

class RWStepShape_RWContextDependentShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ContextDependentShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_ContextDependentShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ContextDependentShapeRepresentation) -> None: ...

class RWStepShape_RWCsgShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_CsgShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_CsgShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_CsgShapeRepresentation) -> None: ...

class RWStepShape_RWCsgSolid:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_CsgSolid) -> None: ...
	def Share(self, ent: StepShape_CsgSolid, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_CsgSolid) -> None: ...

class RWStepShape_RWDefinitionalRepresentationAndShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_DefinitionalRepresentationAndShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_DefinitionalRepresentationAndShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_DefinitionalRepresentationAndShapeRepresentation) -> None: ...

class RWStepShape_RWDimensionalCharacteristicRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_DimensionalCharacteristicRepresentation) -> None: ...
	def Share(self, ent: StepShape_DimensionalCharacteristicRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_DimensionalCharacteristicRepresentation) -> None: ...

class RWStepShape_RWDimensionalLocation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_DimensionalLocation) -> None: ...
	def Share(self, ent: StepShape_DimensionalLocation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_DimensionalLocation) -> None: ...

class RWStepShape_RWDimensionalLocationWithPath:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_DimensionalLocationWithPath) -> None: ...
	def Share(self, ent: StepShape_DimensionalLocationWithPath, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_DimensionalLocationWithPath) -> None: ...

class RWStepShape_RWDimensionalSize:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_DimensionalSize) -> None: ...
	def Share(self, ent: StepShape_DimensionalSize, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_DimensionalSize) -> None: ...

class RWStepShape_RWDimensionalSizeWithPath:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_DimensionalSizeWithPath) -> None: ...
	def Share(self, ent: StepShape_DimensionalSizeWithPath, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_DimensionalSizeWithPath) -> None: ...

class RWStepShape_RWEdge:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Edge) -> None: ...
	def Share(self, ent: StepShape_Edge, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Edge) -> None: ...

class RWStepShape_RWEdgeBasedWireframeModel:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_EdgeBasedWireframeModel) -> None: ...
	def Share(self, ent: StepShape_EdgeBasedWireframeModel, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_EdgeBasedWireframeModel) -> None: ...

class RWStepShape_RWEdgeBasedWireframeShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_EdgeBasedWireframeShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_EdgeBasedWireframeShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_EdgeBasedWireframeShapeRepresentation) -> None: ...

class RWStepShape_RWEdgeCurve:
	def __init__(self) -> None: ...
	def Check(self, ent: StepShape_EdgeCurve, shares: Interface_ShareTool, ach: Interface_Check) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_EdgeCurve) -> None: ...
	def Share(self, ent: StepShape_EdgeCurve, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_EdgeCurve) -> None: ...

class RWStepShape_RWEdgeLoop:
	def __init__(self) -> None: ...
	def Check(self, ent: StepShape_EdgeLoop, shares: Interface_ShareTool, ach: Interface_Check) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_EdgeLoop) -> None: ...
	def Share(self, ent: StepShape_EdgeLoop, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_EdgeLoop) -> None: ...

class RWStepShape_RWExtrudedAreaSolid:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ExtrudedAreaSolid) -> None: ...
	def Share(self, ent: StepShape_ExtrudedAreaSolid, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ExtrudedAreaSolid) -> None: ...

class RWStepShape_RWExtrudedFaceSolid:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ExtrudedFaceSolid) -> None: ...
	def Share(self, ent: StepShape_ExtrudedFaceSolid, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ExtrudedFaceSolid) -> None: ...

class RWStepShape_RWFace:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Face) -> None: ...
	def Share(self, ent: StepShape_Face, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Face) -> None: ...

class RWStepShape_RWFaceBasedSurfaceModel:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_FaceBasedSurfaceModel) -> None: ...
	def Share(self, ent: StepShape_FaceBasedSurfaceModel, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_FaceBasedSurfaceModel) -> None: ...

class RWStepShape_RWFaceBound:
	def __init__(self) -> None: ...
	def Check(self, ent: StepShape_FaceBound, shares: Interface_ShareTool, ach: Interface_Check) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_FaceBound) -> None: ...
	def Share(self, ent: StepShape_FaceBound, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_FaceBound) -> None: ...

class RWStepShape_RWFaceOuterBound:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_FaceOuterBound) -> None: ...
	def Share(self, ent: StepShape_FaceOuterBound, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_FaceOuterBound) -> None: ...

class RWStepShape_RWFaceSurface:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_FaceSurface) -> None: ...
	def Share(self, ent: StepShape_FaceSurface, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_FaceSurface) -> None: ...

class RWStepShape_RWFacetedBrep:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_FacetedBrep) -> None: ...
	def Share(self, ent: StepShape_FacetedBrep, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_FacetedBrep) -> None: ...

class RWStepShape_RWFacetedBrepAndBrepWithVoids:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_FacetedBrepAndBrepWithVoids) -> None: ...
	def Share(self, ent: StepShape_FacetedBrepAndBrepWithVoids, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_FacetedBrepAndBrepWithVoids) -> None: ...

class RWStepShape_RWFacetedBrepShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_FacetedBrepShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_FacetedBrepShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_FacetedBrepShapeRepresentation) -> None: ...

class RWStepShape_RWGeometricCurveSet:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_GeometricCurveSet) -> None: ...
	def Share(self, ent: StepShape_GeometricCurveSet, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_GeometricCurveSet) -> None: ...

class RWStepShape_RWGeometricSet:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_GeometricSet) -> None: ...
	def Share(self, ent: StepShape_GeometricSet, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_GeometricSet) -> None: ...

class RWStepShape_RWGeometricallyBoundedSurfaceShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_GeometricallyBoundedSurfaceShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_GeometricallyBoundedSurfaceShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_GeometricallyBoundedSurfaceShapeRepresentation) -> None: ...

class RWStepShape_RWGeometricallyBoundedWireframeShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_GeometricallyBoundedWireframeShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_GeometricallyBoundedWireframeShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_GeometricallyBoundedWireframeShapeRepresentation) -> None: ...

class RWStepShape_RWHalfSpaceSolid:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_HalfSpaceSolid) -> None: ...
	def Share(self, ent: StepShape_HalfSpaceSolid, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_HalfSpaceSolid) -> None: ...

class RWStepShape_RWLimitsAndFits:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_LimitsAndFits) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_LimitsAndFits) -> None: ...

class RWStepShape_RWLoop:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Loop) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Loop) -> None: ...

class RWStepShape_RWLoopAndPath:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_LoopAndPath) -> None: ...
	def Share(self, ent: StepShape_LoopAndPath, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_LoopAndPath) -> None: ...

class RWStepShape_RWManifoldSolidBrep:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ManifoldSolidBrep) -> None: ...
	def Share(self, ent: StepShape_ManifoldSolidBrep, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ManifoldSolidBrep) -> None: ...

class RWStepShape_RWManifoldSurfaceShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ManifoldSurfaceShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_ManifoldSurfaceShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ManifoldSurfaceShapeRepresentation) -> None: ...

class RWStepShape_RWMeasureQualification:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_MeasureQualification) -> None: ...
	def Share(self, ent: StepShape_MeasureQualification, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_MeasureQualification) -> None: ...

class RWStepShape_RWMeasureRepresentationItemAndQualifiedRepresentationItem:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_MeasureRepresentationItemAndQualifiedRepresentationItem) -> None: ...
	def Share(self, ent: StepShape_MeasureRepresentationItemAndQualifiedRepresentationItem, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_MeasureRepresentationItemAndQualifiedRepresentationItem) -> None: ...

class RWStepShape_RWNonManifoldSurfaceShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_NonManifoldSurfaceShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_NonManifoldSurfaceShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_NonManifoldSurfaceShapeRepresentation) -> None: ...

class RWStepShape_RWOpenShell:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_OpenShell) -> None: ...
	def Share(self, ent: StepShape_OpenShell, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_OpenShell) -> None: ...

class RWStepShape_RWOrientedClosedShell:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_OrientedClosedShell) -> None: ...
	def Share(self, ent: StepShape_OrientedClosedShell, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_OrientedClosedShell) -> None: ...

class RWStepShape_RWOrientedEdge:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_OrientedEdge) -> None: ...
	def Share(self, ent: StepShape_OrientedEdge, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_OrientedEdge) -> None: ...

class RWStepShape_RWOrientedFace:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_OrientedFace) -> None: ...
	def Share(self, ent: StepShape_OrientedFace, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_OrientedFace) -> None: ...

class RWStepShape_RWOrientedOpenShell:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_OrientedOpenShell) -> None: ...
	def Share(self, ent: StepShape_OrientedOpenShell, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_OrientedOpenShell) -> None: ...

class RWStepShape_RWOrientedPath:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_OrientedPath) -> None: ...
	def Share(self, ent: StepShape_OrientedPath, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_OrientedPath) -> None: ...

class RWStepShape_RWPath:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Path) -> None: ...
	def Share(self, ent: StepShape_Path, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Path) -> None: ...

class RWStepShape_RWPlusMinusTolerance:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_PlusMinusTolerance) -> None: ...
	def Share(self, ent: StepShape_PlusMinusTolerance, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_PlusMinusTolerance) -> None: ...

class RWStepShape_RWPointRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_PointRepresentation) -> None: ...
	def Share(self, ent: StepShape_PointRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_PointRepresentation) -> None: ...

class RWStepShape_RWPolyLoop:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_PolyLoop) -> None: ...
	def Share(self, ent: StepShape_PolyLoop, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_PolyLoop) -> None: ...

class RWStepShape_RWPrecisionQualifier:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_PrecisionQualifier) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_PrecisionQualifier) -> None: ...

class RWStepShape_RWQualifiedRepresentationItem:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_QualifiedRepresentationItem) -> None: ...
	def Share(self, ent: StepShape_QualifiedRepresentationItem, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_QualifiedRepresentationItem) -> None: ...

class RWStepShape_RWRevolvedAreaSolid:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_RevolvedAreaSolid) -> None: ...
	def Share(self, ent: StepShape_RevolvedAreaSolid, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_RevolvedAreaSolid) -> None: ...

class RWStepShape_RWRevolvedFaceSolid:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_RevolvedFaceSolid) -> None: ...
	def Share(self, ent: StepShape_RevolvedFaceSolid, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_RevolvedFaceSolid) -> None: ...

class RWStepShape_RWRightAngularWedge:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_RightAngularWedge) -> None: ...
	def Share(self, ent: StepShape_RightAngularWedge, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_RightAngularWedge) -> None: ...

class RWStepShape_RWRightCircularCone:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_RightCircularCone) -> None: ...
	def Share(self, ent: StepShape_RightCircularCone, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_RightCircularCone) -> None: ...

class RWStepShape_RWRightCircularCylinder:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_RightCircularCylinder) -> None: ...
	def Share(self, ent: StepShape_RightCircularCylinder, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_RightCircularCylinder) -> None: ...

class RWStepShape_RWSeamEdge:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_SeamEdge) -> None: ...
	def Share(self, ent: StepShape_SeamEdge, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_SeamEdge) -> None: ...

class RWStepShape_RWShapeDefinitionRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ShapeDefinitionRepresentation) -> None: ...
	def Share(self, ent: StepShape_ShapeDefinitionRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ShapeDefinitionRepresentation) -> None: ...

class RWStepShape_RWShapeDimensionRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ShapeDimensionRepresentation) -> None: ...
	def Share(self, ent: StepShape_ShapeDimensionRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ShapeDimensionRepresentation) -> None: ...

class RWStepShape_RWShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_ShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ShapeRepresentation) -> None: ...

class RWStepShape_RWShapeRepresentationWithParameters:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ShapeRepresentationWithParameters) -> None: ...
	def Share(self, ent: StepShape_ShapeRepresentationWithParameters, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ShapeRepresentationWithParameters) -> None: ...

class RWStepShape_RWShellBasedSurfaceModel:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ShellBasedSurfaceModel) -> None: ...
	def Share(self, ent: StepShape_ShellBasedSurfaceModel, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ShellBasedSurfaceModel) -> None: ...

class RWStepShape_RWSolidModel:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_SolidModel) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_SolidModel) -> None: ...

class RWStepShape_RWSolidReplica:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_SolidReplica) -> None: ...
	def Share(self, ent: StepShape_SolidReplica, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_SolidReplica) -> None: ...

class RWStepShape_RWSphere:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Sphere) -> None: ...
	def Share(self, ent: StepShape_Sphere, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Sphere) -> None: ...

class RWStepShape_RWSubedge:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Subedge) -> None: ...
	def Share(self, ent: StepShape_Subedge, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Subedge) -> None: ...

class RWStepShape_RWSubface:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Subface) -> None: ...
	def Share(self, ent: StepShape_Subface, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Subface) -> None: ...

class RWStepShape_RWSweptAreaSolid:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_SweptAreaSolid) -> None: ...
	def Share(self, ent: StepShape_SweptAreaSolid, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_SweptAreaSolid) -> None: ...

class RWStepShape_RWSweptFaceSolid:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_SweptFaceSolid) -> None: ...
	def Share(self, ent: StepShape_SweptFaceSolid, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_SweptFaceSolid) -> None: ...

class RWStepShape_RWToleranceValue:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ToleranceValue) -> None: ...
	def Share(self, ent: StepShape_ToleranceValue, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ToleranceValue) -> None: ...

class RWStepShape_RWTopologicalRepresentationItem:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_TopologicalRepresentationItem) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_TopologicalRepresentationItem) -> None: ...

class RWStepShape_RWTorus:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Torus) -> None: ...
	def Share(self, ent: StepShape_Torus, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Torus) -> None: ...

class RWStepShape_RWTransitionalShapeRepresentation:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_TransitionalShapeRepresentation) -> None: ...
	def Share(self, ent: StepShape_TransitionalShapeRepresentation, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_TransitionalShapeRepresentation) -> None: ...

class RWStepShape_RWTypeQualifier:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_TypeQualifier) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_TypeQualifier) -> None: ...

class RWStepShape_RWValueFormatTypeQualifier:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_ValueFormatTypeQualifier) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_ValueFormatTypeQualifier) -> None: ...

class RWStepShape_RWVertex:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_Vertex) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_Vertex) -> None: ...

class RWStepShape_RWVertexLoop:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_VertexLoop) -> None: ...
	def Share(self, ent: StepShape_VertexLoop, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_VertexLoop) -> None: ...

class RWStepShape_RWVertexPoint:
	def __init__(self) -> None: ...
	def ReadStep(self, data: StepData_StepReaderData, num: int, ach: Interface_Check, ent: StepShape_VertexPoint) -> None: ...
	def Share(self, ent: StepShape_VertexPoint, iter: Interface_EntityIterator) -> None: ...
	def WriteStep(self, SW: StepData_StepWriter, ent: StepShape_VertexPoint) -> None: ...

# harray1 classes
# harray2 classes
# hsequence classes




# This file was automatically generated by SWIG (http://www.swig.org).
# Version 4.0.2
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

"""
OSD module, see official documentation at
https://www.opencascade.com/doc/occt-7.4.0/refman/html/package_osd.html
"""

from sys import version_info as _swig_python_version_info
if _swig_python_version_info < (2, 7, 0):
    raise RuntimeError("Python 2.7 or later required")

# Import the low-level C/C++ module
if __package__ or "." in __name__:
    from . import _OSD
else:
    import _OSD

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)


def _swig_setattr_nondynamic_instance_variable(set):
    def set_instance_attr(self, name, value):
        if name == "thisown":
            self.this.own(value)
        elif name == "this":
            set(self, name, value)
        elif hasattr(self, name) and isinstance(getattr(type(self), name), property):
            set(self, name, value)
        else:
            raise AttributeError("You cannot add instance attributes to %s" % self)
    return set_instance_attr


def _swig_setattr_nondynamic_class_variable(set):
    def set_class_attr(cls, name, value):
        if hasattr(cls, name) and not isinstance(getattr(cls, name), property):
            set(cls, name, value)
        else:
            raise AttributeError("You cannot add class attributes to %s" % cls)
    return set_class_attr


def _swig_add_metaclass(metaclass):
    """Class decorator for adding a metaclass to a SWIG wrapped class - a slimmed down version of six.add_metaclass"""
    def wrapper(cls):
        return metaclass(cls.__name__, cls.__bases__, cls.__dict__.copy())
    return wrapper


class _SwigNonDynamicMeta(type):
    """Meta class to enforce nondynamic attributes (no new attributes) for a class"""
    __setattr__ = _swig_setattr_nondynamic_class_variable(type.__setattr__)


class SwigPyIterator(object):
    thisown = property(lambda x: x.this.own(), lambda x, v: x.this.own(v), doc="The membership flag")

    def __init__(self, *args, **kwargs):
        raise AttributeError("No constructor defined - class is abstract")
    __repr__ = _swig_repr
    __swig_destroy__ = _OSD.delete_SwigPyIterator

    def value(self) -> "PyObject *":
        return _OSD.SwigPyIterator_value(self)

    def incr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _OSD.SwigPyIterator_incr(self, n)

    def decr(self, n: "size_t"=1) -> "swig::SwigPyIterator *":
        return _OSD.SwigPyIterator_decr(self, n)

    def distance(self, x: "SwigPyIterator") -> "ptrdiff_t":
        return _OSD.SwigPyIterator_distance(self, x)

    def equal(self, x: "SwigPyIterator") -> "bool":
        return _OSD.SwigPyIterator_equal(self, x)

    def copy(self) -> "swig::SwigPyIterator *":
        return _OSD.SwigPyIterator_copy(self)

    def next(self) -> "PyObject *":
        return _OSD.SwigPyIterator_next(self)

    def __next__(self) -> "PyObject *":
        return _OSD.SwigPyIterator___next__(self)

    def previous(self) -> "PyObject *":
        return _OSD.SwigPyIterator_previous(self)

    def advance(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _OSD.SwigPyIterator_advance(self, n)

    def __eq__(self, x: "SwigPyIterator") -> "bool":
        return _OSD.SwigPyIterator___eq__(self, x)

    def __ne__(self, x: "SwigPyIterator") -> "bool":
        return _OSD.SwigPyIterator___ne__(self, x)

    def __iadd__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _OSD.SwigPyIterator___iadd__(self, n)

    def __isub__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator &":
        return _OSD.SwigPyIterator___isub__(self, n)

    def __add__(self, n: "ptrdiff_t") -> "swig::SwigPyIterator *":
        return _OSD.SwigPyIterator___add__(self, n)

    def __sub__(self, *args) -> "ptrdiff_t":
        return _OSD.SwigPyIterator___sub__(self, *args)
    def __iter__(self):
        return self

# Register SwigPyIterator in _OSD:
_OSD.SwigPyIterator_swigregister(SwigPyIterator)


def _dumps_object(klass):
    """ Overwrite default string output for any wrapped object.
    By default, __repr__ method returns something like:
    <OCC.Core.TopoDS.TopoDS_Shape; proxy of <Swig Object of type 'TopoDS_Shape *' at 0x02BB0758> >
    This is too much verbose.
    We prefer :
    <class 'gp_Pnt'>
    or
    <class 'TopoDS_Shape'>
    """
    klass_name = str(klass.__class__).split(".")[3].split("'")[0]
    repr_string = "<class '" + klass_name + "'"
# for TopoDS_Shape, we also look for the base type
    if klass_name == "TopoDS_Shape":
        if klass.IsNull():
            repr_string += ": Null>"
            return repr_string
        st = klass.ShapeType()
        types = {OCC.Core.TopAbs.TopAbs_VERTEX: "Vertex",
                 OCC.Core.TopAbs.TopAbs_SOLID: "Solid",
                 OCC.Core.TopAbs.TopAbs_EDGE: "Edge",
                 OCC.Core.TopAbs.TopAbs_FACE: "Face",
                 OCC.Core.TopAbs.TopAbs_SHELL: "Shell",
                 OCC.Core.TopAbs.TopAbs_WIRE: "Wire",
                 OCC.Core.TopAbs.TopAbs_COMPOUND: "Compound",
                 OCC.Core.TopAbs.TopAbs_COMPSOLID: "Compsolid"}
        repr_string += "; Type:%s" % types[st]        
    elif hasattr(klass, "IsNull"):
        if klass.IsNull():
            repr_string += "; Null"
    repr_string += ">"
    return repr_string


def process_exception(error: "Standard_Failure", method_name: "std::string", class_name: "std::string") -> "void":
    return _OSD.process_exception(error, method_name, class_name)

from six import with_metaclass
import warnings
from OCC.Wrapper.wrapper_utils import Proxy, deprecated

import OCC.Core.Standard
import OCC.Core.NCollection
import OCC.Core.TCollection

from enum import IntEnum
from OCC.Core.Exception import *

OSD_NoLock = _OSD.OSD_NoLock
OSD_ReadLock = _OSD.OSD_ReadLock
OSD_WriteLock = _OSD.OSD_WriteLock
OSD_ExclusiveLock = _OSD.OSD_ExclusiveLock
OSD_RTLD_LAZY = _OSD.OSD_RTLD_LAZY
OSD_RTLD_NOW = _OSD.OSD_RTLD_NOW
OSD_ReadOnly = _OSD.OSD_ReadOnly
OSD_WriteOnly = _OSD.OSD_WriteOnly
OSD_ReadWrite = _OSD.OSD_ReadWrite
OSD_Unavailable = _OSD.OSD_Unavailable
OSD_SUN = _OSD.OSD_SUN
OSD_DEC = _OSD.OSD_DEC
OSD_SGI = _OSD.OSD_SGI
OSD_NEC = _OSD.OSD_NEC
OSD_MAC = _OSD.OSD_MAC
OSD_PC = _OSD.OSD_PC
OSD_HP = _OSD.OSD_HP
OSD_IBM = _OSD.OSD_IBM
OSD_VAX = _OSD.OSD_VAX
OSD_LIN = _OSD.OSD_LIN
OSD_AIX = _OSD.OSD_AIX
OSD_WDirectory = _OSD.OSD_WDirectory
OSD_WDirectoryIterator = _OSD.OSD_WDirectoryIterator
OSD_WEnvironment = _OSD.OSD_WEnvironment
OSD_WFile = _OSD.OSD_WFile
OSD_WFileNode = _OSD.OSD_WFileNode
OSD_WFileIterator = _OSD.OSD_WFileIterator
OSD_WPath = _OSD.OSD_WPath
OSD_WProcess = _OSD.OSD_WProcess
OSD_WProtection = _OSD.OSD_WProtection
OSD_WHost = _OSD.OSD_WHost
OSD_WDisk = _OSD.OSD_WDisk
OSD_WChronometer = _OSD.OSD_WChronometer
OSD_WTimer = _OSD.OSD_WTimer
OSD_WPackage = _OSD.OSD_WPackage
OSD_WEnvironmentIterator = _OSD.OSD_WEnvironmentIterator
OSD_Unknown = _OSD.OSD_Unknown
OSD_Default = _OSD.OSD_Default
OSD_UnixBSD = _OSD.OSD_UnixBSD
OSD_UnixSystemV = _OSD.OSD_UnixSystemV
OSD_VMS = _OSD.OSD_VMS
OSD_OS2 = _OSD.OSD_OS2
OSD_OSF = _OSD.OSD_OSF
OSD_MacOs = _OSD.OSD_MacOs
OSD_Taligent = _OSD.OSD_Taligent
OSD_WindowsNT = _OSD.OSD_WindowsNT
OSD_LinuxREDHAT = _OSD.OSD_LinuxREDHAT
OSD_Aix = _OSD.OSD_Aix
OSD_FILE = _OSD.OSD_FILE
OSD_DIRECTORY = _OSD.OSD_DIRECTORY
OSD_LINK = _OSD.OSD_LINK
OSD_SOCKET = _OSD.OSD_SOCKET
OSD_UNKNOWN = _OSD.OSD_UNKNOWN
OSD_SignalMode_AsIs = _OSD.OSD_SignalMode_AsIs
OSD_SignalMode_Set = _OSD.OSD_SignalMode_Set
OSD_SignalMode_SetUnhandled = _OSD.OSD_SignalMode_SetUnhandled
OSD_SignalMode_Unset = _OSD.OSD_SignalMode_Unset
OSD_FromBeginning = _OSD.OSD_FromBeginning
OSD_FromHere = _OSD.OSD_FromHere
OSD_FromEnd = _OSD.OSD_FromEnd
OSD_None = _OSD.OSD_None
OSD_R = _OSD.OSD_R
OSD_W = _OSD.OSD_W
OSD_RW = _OSD.OSD_RW
OSD_X = _OSD.OSD_X
OSD_RX = _OSD.OSD_RX
OSD_WX = _OSD.OSD_WX
OSD_RWX = _OSD.OSD_RWX
OSD_D = _OSD.OSD_D
OSD_RD = _OSD.OSD_RD
OSD_WD = _OSD.OSD_WD
OSD_RWD = _OSD.OSD_RWD
OSD_XD = _OSD.OSD_XD
OSD_RXD = _OSD.OSD_RXD
OSD_WXD = _OSD.OSD_WXD
OSD_RWXD = _OSD.OSD_RWXD


class OSD_LockType(IntEnum):
	OSD_NoLock = 0
	OSD_ReadLock = 1
	OSD_WriteLock = 2
	OSD_ExclusiveLock = 3
OSD_NoLock = OSD_LockType.OSD_NoLock
OSD_ReadLock = OSD_LockType.OSD_ReadLock
OSD_WriteLock = OSD_LockType.OSD_WriteLock
OSD_ExclusiveLock = OSD_LockType.OSD_ExclusiveLock

class OSD_LoadMode(IntEnum):
	OSD_RTLD_LAZY = 0
	OSD_RTLD_NOW = 1
OSD_RTLD_LAZY = OSD_LoadMode.OSD_RTLD_LAZY
OSD_RTLD_NOW = OSD_LoadMode.OSD_RTLD_NOW

class OSD_OpenMode(IntEnum):
	OSD_ReadOnly = 0
	OSD_WriteOnly = 1
	OSD_ReadWrite = 2
OSD_ReadOnly = OSD_OpenMode.OSD_ReadOnly
OSD_WriteOnly = OSD_OpenMode.OSD_WriteOnly
OSD_ReadWrite = OSD_OpenMode.OSD_ReadWrite

class OSD_OEMType(IntEnum):
	OSD_Unavailable = 0
	OSD_SUN = 1
	OSD_DEC = 2
	OSD_SGI = 3
	OSD_NEC = 4
	OSD_MAC = 5
	OSD_PC = 6
	OSD_HP = 7
	OSD_IBM = 8
	OSD_VAX = 9
	OSD_LIN = 10
	OSD_AIX = 11
OSD_Unavailable = OSD_OEMType.OSD_Unavailable
OSD_SUN = OSD_OEMType.OSD_SUN
OSD_DEC = OSD_OEMType.OSD_DEC
OSD_SGI = OSD_OEMType.OSD_SGI
OSD_NEC = OSD_OEMType.OSD_NEC
OSD_MAC = OSD_OEMType.OSD_MAC
OSD_PC = OSD_OEMType.OSD_PC
OSD_HP = OSD_OEMType.OSD_HP
OSD_IBM = OSD_OEMType.OSD_IBM
OSD_VAX = OSD_OEMType.OSD_VAX
OSD_LIN = OSD_OEMType.OSD_LIN
OSD_AIX = OSD_OEMType.OSD_AIX

class OSD_WhoAmI(IntEnum):
	OSD_WDirectory = 0
	OSD_WDirectoryIterator = 1
	OSD_WEnvironment = 2
	OSD_WFile = 3
	OSD_WFileNode = 4
	OSD_WFileIterator = 5
	OSD_WPath = 6
	OSD_WProcess = 7
	OSD_WProtection = 8
	OSD_WHost = 9
	OSD_WDisk = 10
	OSD_WChronometer = 11
	OSD_WTimer = 12
	OSD_WPackage = 13
	OSD_WEnvironmentIterator = 14
OSD_WDirectory = OSD_WhoAmI.OSD_WDirectory
OSD_WDirectoryIterator = OSD_WhoAmI.OSD_WDirectoryIterator
OSD_WEnvironment = OSD_WhoAmI.OSD_WEnvironment
OSD_WFile = OSD_WhoAmI.OSD_WFile
OSD_WFileNode = OSD_WhoAmI.OSD_WFileNode
OSD_WFileIterator = OSD_WhoAmI.OSD_WFileIterator
OSD_WPath = OSD_WhoAmI.OSD_WPath
OSD_WProcess = OSD_WhoAmI.OSD_WProcess
OSD_WProtection = OSD_WhoAmI.OSD_WProtection
OSD_WHost = OSD_WhoAmI.OSD_WHost
OSD_WDisk = OSD_WhoAmI.OSD_WDisk
OSD_WChronometer = OSD_WhoAmI.OSD_WChronometer
OSD_WTimer = OSD_WhoAmI.OSD_WTimer
OSD_WPackage = OSD_WhoAmI.OSD_WPackage
OSD_WEnvironmentIterator = OSD_WhoAmI.OSD_WEnvironmentIterator

class OSD_SysType(IntEnum):
	OSD_Unknown = 0
	OSD_Default = 1
	OSD_UnixBSD = 2
	OSD_UnixSystemV = 3
	OSD_VMS = 4
	OSD_OS2 = 5
	OSD_OSF = 6
	OSD_MacOs = 7
	OSD_Taligent = 8
	OSD_WindowsNT = 9
	OSD_LinuxREDHAT = 10
	OSD_Aix = 11
OSD_Unknown = OSD_SysType.OSD_Unknown
OSD_Default = OSD_SysType.OSD_Default
OSD_UnixBSD = OSD_SysType.OSD_UnixBSD
OSD_UnixSystemV = OSD_SysType.OSD_UnixSystemV
OSD_VMS = OSD_SysType.OSD_VMS
OSD_OS2 = OSD_SysType.OSD_OS2
OSD_OSF = OSD_SysType.OSD_OSF
OSD_MacOs = OSD_SysType.OSD_MacOs
OSD_Taligent = OSD_SysType.OSD_Taligent
OSD_WindowsNT = OSD_SysType.OSD_WindowsNT
OSD_LinuxREDHAT = OSD_SysType.OSD_LinuxREDHAT
OSD_Aix = OSD_SysType.OSD_Aix

class OSD_KindFile(IntEnum):
	OSD_FILE = 0
	OSD_DIRECTORY = 1
	OSD_LINK = 2
	OSD_SOCKET = 3
	OSD_UNKNOWN = 4
OSD_FILE = OSD_KindFile.OSD_FILE
OSD_DIRECTORY = OSD_KindFile.OSD_DIRECTORY
OSD_LINK = OSD_KindFile.OSD_LINK
OSD_SOCKET = OSD_KindFile.OSD_SOCKET
OSD_UNKNOWN = OSD_KindFile.OSD_UNKNOWN

class OSD_SignalMode(IntEnum):
	OSD_SignalMode_AsIs = 0
	OSD_SignalMode_Set = 1
	OSD_SignalMode_SetUnhandled = 2
	OSD_SignalMode_Unset = 3
OSD_SignalMode_AsIs = OSD_SignalMode.OSD_SignalMode_AsIs
OSD_SignalMode_Set = OSD_SignalMode.OSD_SignalMode_Set
OSD_SignalMode_SetUnhandled = OSD_SignalMode.OSD_SignalMode_SetUnhandled
OSD_SignalMode_Unset = OSD_SignalMode.OSD_SignalMode_Unset

class OSD_FromWhere(IntEnum):
	OSD_FromBeginning = 0
	OSD_FromHere = 1
	OSD_FromEnd = 2
OSD_FromBeginning = OSD_FromWhere.OSD_FromBeginning
OSD_FromHere = OSD_FromWhere.OSD_FromHere
OSD_FromEnd = OSD_FromWhere.OSD_FromEnd

class OSD_SingleProtection(IntEnum):
	OSD_None = 0
	OSD_R = 1
	OSD_W = 2
	OSD_RW = 3
	OSD_X = 4
	OSD_RX = 5
	OSD_WX = 6
	OSD_RWX = 7
	OSD_D = 8
	OSD_RD = 9
	OSD_WD = 10
	OSD_RWD = 11
	OSD_XD = 12
	OSD_RXD = 13
	OSD_WXD = 14
	OSD_RWXD = 15
OSD_None = OSD_SingleProtection.OSD_None
OSD_R = OSD_SingleProtection.OSD_R
OSD_W = OSD_SingleProtection.OSD_W
OSD_RW = OSD_SingleProtection.OSD_RW
OSD_X = OSD_SingleProtection.OSD_X
OSD_RX = OSD_SingleProtection.OSD_RX
OSD_WX = OSD_SingleProtection.OSD_WX
OSD_RWX = OSD_SingleProtection.OSD_RWX
OSD_D = OSD_SingleProtection.OSD_D
OSD_RD = OSD_SingleProtection.OSD_RD
OSD_WD = OSD_SingleProtection.OSD_WD
OSD_RWD = OSD_SingleProtection.OSD_RWD
OSD_XD = OSD_SingleProtection.OSD_XD
OSD_RXD = OSD_SingleProtection.OSD_RXD
OSD_WXD = OSD_SingleProtection.OSD_WXD
OSD_RWXD = OSD_SingleProtection.OSD_RWXD


@classnotwrapped
class OSD_Timer:
	pass

@classnotwrapped
class OSD_PerfMeter:
	pass

@classnotwrapped
class OSD_Disk:
	pass

@classnotwrapped
class OSD_Protection:
	pass

@classnotwrapped
class OSD_MemInfo:
	pass

@classnotwrapped
class OSD_DirectoryIterator:
	pass

@classnotwrapped
class OSD_Chronometer:
	pass

@classnotwrapped
class OSD_ThreadPool:
	pass

@classnotwrapped
class OSD_SharedLibrary:
	pass

@classnotwrapped
class OSD_Error:
	pass

@classnotwrapped
class OSD_Host:
	pass

@classnotwrapped
class OSD_Parallel:
	pass

@classnotwrapped
class OSD_Directory:
	pass

@classnotwrapped
class OSD_Path:
	pass

@classnotwrapped
class OSD_Process:
	pass

@classnotwrapped
class OSD_MAllocHook:
	pass

@classnotwrapped
class OSD_FileNode:
	pass

@classnotwrapped
class OSD_File:
	pass

@classnotwrapped
class OSD_FileIterator:
	pass

@classnotwrapped
class OSD_Thread:
	pass

@classnotwrapped
class OSD_Environment:
	pass

@classnotwrapped
class OSD:
	pass









import logging

# backend constants
WX = "wx"
PYSIDE = "qt-pyside"
PYSIDE2 = "qt-pyside2"
PYQT4 = "qt-pyqt4"
PYQT5 = "qt-pyqt5"

# backend module
HAVE_PYQT5, HAVE_PYQT4, HAVE_PYSIDE, HAVE_PYSIDE2, HAVE_WX = False, False, False, False, False

# is any backend imported?
HAVE_BACKEND = False
BACKEND_MODULE = "No backend loaded"

log = logging.getLogger(__name__)
log.setLevel(logging.DEBUG)


def load_pyqt5():
    """ returns True is PyQt5 found, else False
    """
    global HAVE_PYQT5, QtCore, QtGui, QtWidgets, QtOpenGL

    # backend already loaded, dont load another one
    if loaded_backend():
        return False

    try:
        from PyQt5 import QtCore, QtGui, QtOpenGL, QtWidgets
        HAVE_PYQT5 = True
    except ImportError:
        HAVE_PYQT5 = False
    return HAVE_PYQT5


def load_pyqt4():
    """ returns True is PyQt4 found, else False
    """
    global HAVE_PYQT4, QtCore, QtGui, QtWidgets, QtOpenGL

    # backend already loaded, dont load another one
    if loaded_backend():
        return False

    try:
        from PyQt4 import QtCore, QtGui, QtOpenGL
        QtWidgets = QtGui
        HAVE_PYQT4 = True
    except ImportError:
        HAVE_PYQT4 = False
    return HAVE_PYQT4


def load_pyside():
    """ returns True is PySide found, else False
    """
    global HAVE_PYSIDE, QtCore, QtGui, QtWidgets, QtOpenGL

    # backend already loaded, dont load another one
    if loaded_backend():
        return False

    try:
        from PySide import QtCore, QtGui, QtOpenGL
        QtWidgets = QtGui
        HAVE_PYSIDE = True
    except ImportError:
        HAVE_PYSIDE = False
    return HAVE_PYSIDE


def load_pyside2():
    """ returns True is PySide2 found, else False
    """
    global HAVE_PYSIDE2, QtCore, QtGui, QtWidgets, QtOpenGL

    # backend already loaded, dont load another one
    if loaded_backend():
        return False

    try:
        from PySide2 import QtCore, QtGui, QtOpenGL, QtWidgets
        HAVE_PYSIDE2 = True
    except ImportError:
        HAVE_PYSIDE2 = False
    return HAVE_PYSIDE2


def load_wx():
    """ returns True is wxPython found, else False
    """

    # backend already loaded, dont load another one
    if loaded_backend():
        return False

    global HAVE_WX
    try:
        import wx
        HAVE_WX = True
    except ImportError:
        HAVE_WX = False
    return HAVE_WX


def loaded_backend():
    return HAVE_BACKEND


def get_loaded_backend():
    return BACKEND_MODULE


def load_any_qt_backend():
    """ loads any qt based backend. First try to load
    PyQt5, then PyQt4 and finally PySide. Raise an exception
    if none of them are available
    """
    pyqt5_loaded = False
    pyqt4_loaded = False
    pyside_loaded = False
    # by default, load PyQt5
    pyqt5_loaded = load_backend(PYQT5)
    if not pyqt5_loaded:
        # load pyqt4
        pyqt4_loaded = load_backend(PYQT4)
        # finally try to load pyside
        if not pyqt4_loaded:
            pyside2_loaded = load_backend(PYSIDE2)
            if not pyside2_loaded:
                pyside_loaded = load_backend(PYSIDE)
    if not (pyqt5_loaded or pyqt4_loaded or pyside2_loaded or pyside_loaded):
        raise AssertionError("None of the PyQt5, PtQt4, PySide2 or PySide backend can be loaded")
    else:
        return True


def load_backend(backend_str=None):
    """ loads a gui backend

    If no Qt (such as PyQt5, PyQt4 or PySide) backend is found, wx is loaded

    The search order for pythonocc compatible gui modules is:
        PyQt5, PyQt4, PySide2, PySide, Wx

    Note
    ----
    Wx is imported when no Qt backend is found.

    Parameters
    ----------
    backend_str : str

        specifies which backend to load

        backend_str is one of ( "qt-pyqt5", "qt-pyqt4", ""qt-pyside2", qt-pyside", "wx" )

        if no value has been set, load the first module in gui module search
        order

    Returns
    -------
    str
        the name of the loaded backend
        one of ( "qt-pyqt5", "qt-pyqt4", "qt-pyside2", "qt-pyside", "wx" )

    Raises
    ------

    ValueError
        * when a backend is already loaded
        * when an invalid backend_str is specified

    ImportError
        when the backend specified in ``backend_str`` could not be imported

    """
    global HAVE_BACKEND, BACKEND_MODULE

    if HAVE_BACKEND:
        msg = "The {0} backend is already loaded..." \
              "``load_backend`` can only be called once per session".format(BACKEND_MODULE)
        log.info(msg)
        return BACKEND_MODULE

    if backend_str is not None:
        compatible_backends = (PYQT5, PYQT4, PYSIDE2, PYSIDE, WX)
        if not backend_str in compatible_backends:
            msg = "incompatible backend_str specified: {0}\n" \
                  "backend is one of : {1}".format(backend_str,
                                                   compatible_backends)
            log.critical(msg)
            raise ValueError(msg)

    if backend_str == PYQT5 or backend_str is None:
        if load_pyqt5():
            HAVE_BACKEND = True
            BACKEND_MODULE = 'qt-pyqt5'
            log.info("backend loaded: {0}".format(BACKEND_MODULE))
            return BACKEND_MODULE
        if backend_str == PYQT5 and not HAVE_BACKEND:
            msg = "{0} backend could not be loaded".format(backend_str)
            log.exception(msg)
            raise ValueError(msg)
        else:
            pass

    if backend_str == PYQT4 or (backend_str is None and not HAVE_BACKEND):
        if load_pyqt4():
            HAVE_BACKEND = True
            BACKEND_MODULE = 'qt-pyqt4'
            log.info("backend loaded: {0}".format(BACKEND_MODULE))
            return BACKEND_MODULE
        elif backend_str == PYQT4 and not HAVE_BACKEND:
            msg = "{0} backend could not be loaded".format(backend_str)
            log.exception(msg)
            raise ValueError(msg)

        else:
            pass

    if backend_str == PYSIDE2 or (backend_str is None and not HAVE_BACKEND):
        if load_pyside2():
            HAVE_BACKEND = True
            BACKEND_MODULE = 'qt-pyside2'
            log.info("backend loaded: {0}".format(BACKEND_MODULE))
            return BACKEND_MODULE
        elif backend_str == PYSIDE2 and not HAVE_BACKEND:
            msg = "{0} could not be loaded".format(backend_str)
            log.exception(msg)
            raise ValueError(msg)
        else:
            pass

    if backend_str == PYSIDE or (backend_str is None and not HAVE_BACKEND):
        if load_pyside():
            HAVE_BACKEND = True
            BACKEND_MODULE = 'qt-pyside'
            log.info("backend loaded: {0}".format(BACKEND_MODULE))
            return BACKEND_MODULE
        elif backend_str == PYSIDE and not HAVE_BACKEND:
            msg = "{0} backend could not be loaded".format(backend_str)
            log.exception(msg)
            raise ValueError(msg)
        else:
            pass

    if backend_str == WX or (backend_str is None and not HAVE_BACKEND):
        if load_wx():
            HAVE_BACKEND = True
            BACKEND_MODULE = 'wx'
            log.info("backend loaded: {0}".format(BACKEND_MODULE))
            return BACKEND_MODULE
        elif backend_str == WX and not HAVE_BACKEND:
            msg = "{0} backend could not be loaded".format(backend_str)
            log.exception(msg)
            raise ValueError(msg)
        else:
            pass

    if not HAVE_BACKEND:
        raise ImportError("No compliant GUI library could be imported.\n"
                          "Either PyQt5, PyQt4, PySide2, PySide, or wxPython "
                          "is required")


def get_qt_modules():
    """

    Returns
    -------
    tuple : ( QtCore, QtGui, QtWidgets, QtOpenGL )
        here QtWidgets shadows QtGui when a PyQt4 or PySide module is loaded
        this is the most coherent way to get PyQt5 compliant code

    Raises
    ------

    ValueError
        when no Qt backend has been yet loaded
        informs the user to call `load_backend` or that no Qt python module
        ( PyQt5, PyQt4 or PySide ) is found

    """
    if not HAVE_BACKEND:
        raise ValueError("no backend has been imported yet with "
                         "``load_backend``... ")

    if HAVE_PYQT5 or HAVE_PYQT4 or HAVE_PYSIDE2 or HAVE_PYSIDE:
        return QtCore, QtGui, QtWidgets, QtOpenGL
    elif HAVE_WX:
        raise ValueError("the Wx backend is already loaded")
    else:
        msg = ("no Qt backend is loaded, hence cannot return any modules\n"
               "either you havent got PyQt5, PyQt4, PySide2 or PySide installed\n"
               "or you havent yet loaded a backend with the "
               "`OCC.Display.backend.load_backend` function")
        raise ValueError(msg)
